@@ -1,4 +1,3 @@
-// [[Rcpp::depends(RcppArmadillo)]]
 #include "alphasimr.h"
 
 // Class for storing recombination history
@@ -58,29 +57,69 @@ arma::Mat<int> RecHist::getHist(arma::uword ind,
 // start, the position downstream to start the gamma process (should be a negative value)
 // end, the length of the interval used to sample
 // v, the interference parameter
+// p, the proportion of non-interfering crossovers
 // n, the number of gamma deviates sampled at a time (affects performance, not results)
 arma::vec sampleChiasmata(double start, double end, double v, 
-                          arma::uword n=40){
-  // Sample deviates from a gamma distribution
-  arma::vec output = arma::randg<arma::vec>(n, arma::distr_param(v,0.5/v));
-  
-  // Find locations on genetic map
-  output = cumsum(output)+start;
-  
-  // Add additional values if max position less than end
-  while(output(output.n_elem-1)<end){
-    arma::vec tmp = arma::randg<arma::vec>(n, arma::distr_param(v,0.5/v));
-    tmp = cumsum(tmp) + output(output.n_elem-1);
-    output = join_cols(output, tmp);
+                          double p, arma::uword n=40){
+  if((1-p)>1e-6){ // Gamma model
+    // Sample deviates from a gamma distribution
+    arma::vec output = arma::randg<arma::vec>(n, arma::distr_param(v,1.0/(2.0*v)));
+    
+    // Find locations on genetic map
+    output = cumsum(output)+start;
+    
+    // Add additional values if max position less than end
+    while(output(output.n_elem-1)<end){
+      arma::vec tmp = arma::randg<arma::vec>(n, arma::distr_param(v,1.0/(2.0*v)));
+      tmp = cumsum(tmp) + output(output.n_elem-1);
+      output = join_cols(output, tmp);
+    }
+    
+    // Remove values less than 0
+    output = output(find(output>0));
+    
+    // Select values less than the end
+    return output(find(output<end));
+  }else{ // Gamma sprinkling model
+    // Sample type 1 deviates from a gamma distribution
+    arma::vec type1 = arma::randg<arma::vec>(n, arma::distr_param(v,1.0/(2.0*v*(1-p))));
+    
+    // Find locations on genetic map
+    type1 = cumsum(type1)+start;
+    
+    // Add additional values if max position less than end
+    while(type1(type1.n_elem-1)<end){
+      arma::vec tmp = arma::randg<arma::vec>(n, arma::distr_param(v,1.0/(2.0*v*(1-p))));
+      tmp = cumsum(tmp) + type1(type1.n_elem-1);
+      type1 = join_cols(type1, tmp);
+    }
+    
+    // Remove values less than 0
+    type1 = type1(find(type1>0));
+    
+    // Select values less than the end
+    type1 = type1(find(type1<end));
+    
+    // Sample type 2 deviates from a gamma distribution
+    arma::vec type2 = arma::randg<arma::vec>(n, arma::distr_param(1.0,1.0/(2.0*v*p)));
+    
+    // Find locations on genetic map
+    type2 = cumsum(type2);
+    
+    // Add additional values if max position less than end
+    while(type2(type2.n_elem-1)<end){
+      arma::vec tmp = arma::randg<arma::vec>(n, arma::distr_param(1.0,1.0/(2.0*v*p)));
+      tmp = cumsum(tmp) + type2(type2.n_elem-1);
+      type2 = join_cols(type2, tmp);
+    }
+    
+    // Select values less than the end
+    type2 = type2(find(type2<end));
+    
+    // Return sorted vector of type 1 and type 2 crossovers
+    return sort(join_cols(type1, type2));
   }
-  
-  // Remove values less than 0
-  output = output(find(output>0));
-  
-  // Return values less than the end
-  return output(find(output<end));
 }
-
 // Samples the locations for chiasmata via a gamma process for a quadrivalent
 // CO interference is assumed to occur between all arms
 // The first arm is sampled at random
@@ -88,20 +127,21 @@ arma::vec sampleChiasmata(double start, double end, double v,
 // exchange, the positions where chromosomes switch
 // end, the length of the interval used to sample
 // v, the interference parameter
+// p, the proportion of non-interfering crossovers
 // n1, the number of gamma deviates sampled for the first arm 
 // n2, the number of gamma deviates sampled for all other arms
 arma::field<arma::vec> sampleQuadChiasmata(double start, double exchange, double end, 
-                                           double v, arma::uword n1=40, arma::uword n2=8){
+                                           double v, double p, arma::uword n1=40, arma::uword n2=8){
   arma::field<arma::vec> output(4);
   arma::vec u(1, arma::fill::randu);
   
   // Randomly set order of chromosome arms
   arma::uvec arm = {0, 1, 2, 3};
   arm = shuffle(arm);
-  double nearest, terminator, p;
+  double nearest, terminator, prob;
   
   // First arm
-  output(arm(0)) = arma::randg<arma::vec>(n1, arma::distr_param(v,0.5/v));
+  output(arm(0)) = arma::randg<arma::vec>(n1, arma::distr_param(v,1.0/(2.0*v*(1-p))));
   output(arm(0)) = cumsum(output(arm(0))) + start;
   if(arm(0)%2){ // Tail
     terminator = end - exchange;
@@ -109,7 +149,7 @@ arma::field<arma::vec> sampleQuadChiasmata(double start, double exchange, double
     terminator = exchange;
   }
   while( output(arm(0))(output(arm(0)).n_elem-1) < terminator ){
-    arma::vec tmp = arma::randg<arma::vec>(n2, arma::distr_param(v,0.5/v));
+    arma::vec tmp = arma::randg<arma::vec>(n2, arma::distr_param(v,1.0/(2.0*v*(1-p))));
     tmp = cumsum(tmp) + output(arm(0))(output(arm(0)).n_elem-1);
     output(arm(0)) = join_cols(output(arm(0)), tmp);
   }
@@ -123,14 +163,14 @@ arma::field<arma::vec> sampleQuadChiasmata(double start, double exchange, double
   // All other arms
   for(arma::uword i=1; i<4; ++i){
     output(arm(i)).set_size(1+n2);
-    p = R::pgamma(nearest, v, 0.5/v, 1, 0);
+    prob = R::pgamma(nearest, v, 1.0/(2.0*v*(1-p)), 1, 0);
     u.randu();
-    u(0) = u(0)*(1-p)+p;
-    output(arm(i))(0) = R::qgamma(u(0), v, 0.5/v, 1, 0) - nearest;
+    u(0) = u(0)*(1-prob)+prob;
+    output(arm(i))(0) = R::qgamma(u(0), v, 1.0/(2.0*v*(1-p)), 1, 0) - nearest;
     if(output(arm(i))(0) < nearest){
       nearest = output(arm(i))(0);
     }
-    output(arm(i))(arma::span(1,n2)) = arma::randg<arma::vec>(n2, arma::distr_param(v,0.5/v));
+    output(arm(i))(arma::span(1,n2)) = arma::randg<arma::vec>(n2, arma::distr_param(v,1.0/(2.0*v*(1-p))));
     output(arm(i)) = cumsum(output(arm(i)));
     if(arm(i)%2){ // Tail
       terminator = end - exchange;
@@ -138,7 +178,7 @@ arma::field<arma::vec> sampleQuadChiasmata(double start, double exchange, double
       terminator = exchange;
     }
     while( output(arm(i))(output(arm(i)).n_elem-1) < terminator ){
-      arma::vec tmp = arma::randg<arma::vec>(n2, arma::distr_param(v,0.5/v));
+      arma::vec tmp = arma::randg<arma::vec>(n2, arma::distr_param(v,1.0/(2.0*v*(1-p))));
       tmp = cumsum(tmp) + output(arm(i))(output(arm(i)).n_elem-1);
       output(arm(i)) = join_cols(output(arm(i)), tmp);
     }
@@ -149,6 +189,41 @@ arma::field<arma::vec> sampleQuadChiasmata(double start, double exchange, double
       output(arm(i)) = sort(exchange-output(arm(i)));
     }
   }
+  
+  if(p>1.0e-6){ // Sprinkle recombinations
+    for(arma::uword i=0; i<4; ++i){
+      if(arm(i)%2){ // Tail
+        terminator = end - exchange;
+      }else{ // Head
+        terminator = exchange;
+      }
+      
+      // Sample type 2 deviates from a gamma distribution
+      arma::vec type2 = arma::randg<arma::vec>(n2, arma::distr_param(1.0,1.0/(2.0*v*p)));
+      
+      // Find locations on genetic map
+      type2 = cumsum(type2);
+      
+      // Add additional values if max position less than terminator
+      while(type2(type2.n_elem-1)<terminator){
+        arma::vec tmp = arma::randg<arma::vec>(n2, arma::distr_param(1.0,1.0/(2.0*v*p)));
+        tmp = cumsum(tmp) + type2(type2.n_elem-1);
+        type2 = join_cols(type2, tmp);
+      }
+      
+      // Select values less than the end
+      type2 = type2(find(type2<terminator));
+      
+      // Add exchange to genetic position if in tail
+      if(arm(i)%2){ // Tail
+        type2 += exchange;
+      }
+      
+      // Combine type 1 and 2 crossovers and sort
+      output(arm(i)) = sort(join_cols(output(arm(i)), type2));
+    }
+  }
+  
   return output;
 }
 
@@ -232,7 +307,7 @@ arma::Mat<int> removeDoubleCO(const arma::Mat<int>& X){
 }
 
 // Finds recombination map for a bivalent pair
-arma::Mat<int> findBivalentCO(const arma::vec& genMap, double v){
+arma::Mat<int> findBivalentCO(const arma::vec& genMap, double v, double p){
   arma::uword startPos=0, endPos, readChr=0, nCO;
   double genLen = genMap(genMap.n_elem-1);
   
@@ -241,7 +316,7 @@ arma::Mat<int> findBivalentCO(const arma::vec& genMap, double v){
   double start = u(0)-10;
   
   // Find crossover positions
-  arma::vec posCO = sampleChiasmata(start, genLen, v);
+  arma::vec posCO = sampleChiasmata(start, genLen, v, p);
   if(posCO.n_elem==0){
     arma::Mat<int> output(1,2,arma::fill::ones);
     return output;
@@ -284,7 +359,8 @@ arma::Mat<int> findBivalentCO(const arma::vec& genMap, double v){
  * The second centromere is sampled at random
  */
 arma::field<arma::Mat<int> > findQuadrivalentCO(const arma::vec& genMap,
-                                                double centromere, double v){
+                                                double centromere, double v,
+                                                double p){
   arma::field<arma::Mat<int> > output(2);
   double genLen = genMap(genMap.n_elem-1);
   
@@ -295,7 +371,7 @@ arma::field<arma::Mat<int> > findQuadrivalentCO(const arma::vec& genMap,
   double start = u(0)-10;
   
   // Determine crossover postions
-  arma::field<arma::vec> posCO = sampleQuadChiasmata(start, exchange, genLen, v);
+  arma::field<arma::vec> posCO = sampleQuadChiasmata(start, exchange, genLen, v, p);
   
   // Set chromatid configuration for chiasmata
   arma::field<arma::umat> chromatidPairs(4);
@@ -720,9 +796,10 @@ void bivalent(const arma::Col<unsigned char>& chr1,
               const arma::Col<unsigned char>& chr2,
               const arma::vec& genMap,
               double v,
+              double p,
               arma::Col<unsigned char>& output,
               arma::Mat<int>& hist){
-  hist = findBivalentCO(genMap, v);
+  hist = findBivalentCO(genMap, v, p);
   if(hist.n_rows==1){
     output = chr1;
   }else{
@@ -764,6 +841,7 @@ void quadrivalent(const arma::Col<unsigned char>& chr1,
                   const arma::vec& genMap,
                   double centromere,
                   double v,
+                  double p,
                   arma::Col<unsigned char>& output1,
                   arma::Col<unsigned char>& output2,
                   arma::Mat<int>& hist1,
@@ -771,7 +849,7 @@ void quadrivalent(const arma::Col<unsigned char>& chr1,
   int nBins = chr1.n_elem;
   
   arma::field<arma::Mat<int> > output;
-  output = findQuadrivalentCO(genMap, centromere, v);
+  output = findQuadrivalentCO(genMap, centromere, v, p);
   
   hist1 = output(0);
   hist2 = output(1);
@@ -909,7 +987,8 @@ void quadrivalent(const arma::Col<unsigned char>& chr1,
 // trackRec: track recombination
 // motherPloidy: ploidy level of mother 
 // fatherPloidy: ploidy level of father
-// v: interference paramater for gamma model
+// v: interference parameter for gamma model
+// p: proportion of non-interferring crossovers
 // quadProb: probability of quadrivalent formation
 // nThreads: number of threads for parallel computing
 // [[Rcpp::export]]
@@ -924,6 +1003,7 @@ Rcpp::List cross(
     arma::uword motherPloidy,
     arma::uword fatherPloidy,
     double v,
+    double p,
     const arma::vec& motherCentromere,
     const arma::vec& fatherCentromere,
     double quadProb,
@@ -939,6 +1019,9 @@ Rcpp::List cross(
   if(trackRec){
     hist.setSize(nInd,nChr,ploidy);
   }
+  if(nChr<nThreads){
+    nThreads = nChr;
+  }
   //Loop through chromosomes
 #ifdef _OPENMP
 #pragma omp parallel for schedule(static) num_threads(nThreads)
@@ -972,6 +1055,7 @@ Rcpp::List cross(
                      motherGeno(chr).slice(mother(ind)).col(xm(x+1)),
                      femaleMap(chr),
                      v,
+                     p,
                      gamete1,
                      hist1);
             tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -988,6 +1072,7 @@ Rcpp::List cross(
                      motherGeno(chr).slice(mother(ind)).col(xm(x+3)),
                      femaleMap(chr),
                      v,
+                     p,
                      gamete1,
                      hist1);
             tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -1007,6 +1092,7 @@ Rcpp::List cross(
                          femaleMap(chr),
                          motherCentromere(chr),
                          v,
+                         p,
                          gamete1,
                          gamete2,
                          hist1,
@@ -1031,6 +1117,7 @@ Rcpp::List cross(
                    motherGeno(chr).slice(mother(ind)).col(xm(x+1)),
                    femaleMap(chr),
                    v,
+                   p,
                    gamete1,
                    hist1);
           tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -1055,6 +1142,7 @@ Rcpp::List cross(
                      fatherGeno(chr).slice(father(ind)).col(xf(x+1)),
                      maleMap(chr),
                      v,
+                     p,
                      gamete1,
                      hist1);
             tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -1071,6 +1159,7 @@ Rcpp::List cross(
                      fatherGeno(chr).slice(father(ind)).col(xf(x+3)),
                      maleMap(chr),
                      v,
+                     p,
                      gamete1,
                      hist1);
             tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -1090,6 +1179,7 @@ Rcpp::List cross(
                          maleMap(chr),
                          fatherCentromere(chr),
                          v,
+                         p,
                          gamete1,
                          gamete2,
                          hist1,
@@ -1114,6 +1204,7 @@ Rcpp::List cross(
                    fatherGeno(chr).slice(father(ind)).col(xf(x+1)),
                    maleMap(chr),
                    v,
+                   p,
                    gamete1,
                    hist1);
           tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -1141,7 +1232,7 @@ Rcpp::List cross(
 Rcpp::List createDH2(
     const arma::field<arma::Cube<unsigned char> >& geno, 
     arma::uword nDH, const arma::field<arma::vec>& genMap, 
-    double v, bool trackRec, int nThreads){
+    double v, double p, bool trackRec, int nThreads){
   arma::uword nChr = geno.n_elem;
   arma::uword nInd = geno(0).n_slices;
   //Output data
@@ -1150,6 +1241,9 @@ Rcpp::List createDH2(
   if(trackRec){
     hist.setSize(nInd*nDH,nChr,2);
   }
+  if(nChr<nThreads){
+    nThreads = nChr;
+  }
 #ifdef _OPENMP
 #pragma omp parallel for schedule(static) num_threads(nThreads)
 #endif
@@ -1166,6 +1260,7 @@ Rcpp::List createDH2(
                  geno(chr).slice(ind).col(x(1)),
                  genMap(chr),
                  v,
+                 p,
                  gamete,
                  histMat);
         for(arma::uword j=0; j<2; ++j){ //ploidy loop
@@ -1193,7 +1288,7 @@ Rcpp::List createDH2(
 Rcpp::List createReducedGenome(
     const arma::field<arma::Cube<unsigned char> >& geno, 
     arma::uword nProgeny, const arma::field<arma::vec>& genMap, 
-    double v, bool trackRec, arma::uword ploidy,  
+    double v, double p, bool trackRec, arma::uword ploidy,  
     arma::vec& centromere, double quadProb, int nThreads){
   arma::uword nChr = geno.n_elem;
   arma::uword nInd = geno(0).n_slices;
@@ -1203,6 +1298,9 @@ Rcpp::List createReducedGenome(
   if(trackRec){
     hist.setSize(nInd*nProgeny,nChr,ploidy/2);
   }
+  if(nChr<nThreads){
+    nThreads = nChr;
+  }
 #ifdef _OPENMP
 #pragma omp parallel for schedule(static) num_threads(nThreads)
 #endif
@@ -1228,6 +1326,7 @@ Rcpp::List createReducedGenome(
                      geno(chr).slice(par).col(x(y+1)),
                      genMap(chr),
                      v,
+                     p,
                      gamete1,
                      hist1);
             tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -1244,6 +1343,7 @@ Rcpp::List createReducedGenome(
                      geno(chr).slice(par).col(x(y+3)),
                      genMap(chr),
                      v,
+                     p,
                      gamete1,
                      hist1);
             tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -1263,6 +1363,7 @@ Rcpp::List createReducedGenome(
                          genMap(chr),
                          centromere(chr),
                          v,
+                         p,
                          gamete1,
                          gamete2,
                          hist1,
@@ -1287,6 +1388,7 @@ Rcpp::List createReducedGenome(
                    geno(chr).slice(par).col(x(y+1)),
                    genMap(chr),
                    v,
+                   p,
                    gamete1,
                    hist1);
           tmpGeno.slice(ind).col(progenyChr) = gamete1;
@@ -1308,119 +1410,3 @@ Rcpp::List createReducedGenome(
   }
   return Rcpp::List::create(Rcpp::Named("geno")=output);
 }
-
-// Converts    recHist (recombinations between generations) to
-//          ibdRecHist (recombinations since the base generation/population)
-// [[Rcpp::export]]
-Rcpp::List getIbdRecHist(const Rcpp::List          & recHist,
-                         const Rcpp::IntegerMatrix & pedigree,
-                         const Rcpp::IntegerVector & nLociPerChr) {
-  // This is an utterly complicated function! There has to be a neater way to do this. Gregor
-  RecHist ibdRecHist;
-  arma::uword nInd = pedigree.nrow();
-  arma::uword nChr = nLociPerChr.size();
-  ibdRecHist.setSize(nInd, nChr, 2);
-  for (arma::uword ind = 0; ind < nInd; ++ind) {
-    Rcpp::List recHistInd = recHist(ind);
-    for (arma::uword par = 0; par < 2; ++par) {
-      int pId = pedigree(ind, par);
-      if (pId == 0) { // Individual is     a founder --> set founder gamete code
-        for (arma::uword chr = 0; chr < nChr; ++chr) {
-          if (0 < nLociPerChr(chr)) {
-            arma::Mat<int> recHistIndChrPar;
-            recHistIndChrPar.set_size(1, 2);
-            recHistIndChrPar(0, 0) = 2 * (ind + 1) - 1 + par;
-            recHistIndChrPar(0, 1) = 1;
-            ibdRecHist.addHist(recHistIndChrPar, ind, chr, par);
-          }
-        }
-      } else {        // Individual is not a founder --> get founder gamete code & recombinations
-        pId -= 1; // R to C++ indexing
-        Rcpp::List recHistPar = recHist(pId);
-        for (arma::uword chr = 0; chr < nChr; ++chr) {
-          if (0 < nLociPerChr(chr)) {
-            Rcpp::List recHistIndChr = recHistInd(chr);
-            arma::Mat<int> recHistIndChrPar = recHistIndChr(par);
-            arma::uword nRecSegInd = recHistIndChrPar.n_rows;
-            if (recHistPar.size() == 0) { // Parent is     a founder and has no recHist info --> get founder gamete codes and put them onto individual recombinations
-              for (arma::uword recSegInd = 0; recSegInd < nRecSegInd; ++recSegInd) {
-                int source = recHistIndChrPar(recSegInd, 0) - 1;
-                recHistIndChrPar(recSegInd, 0) = ibdRecHist.getHist(pId, chr, source)(0, 0);
-              }
-              ibdRecHist.addHist(recHistIndChrPar, ind, chr, par);
-            } else {                      // Parent is not a founder and has    recHist info --> parse and combine parent and individual recombinations
-              // Parent's all ancestral recombinations
-              arma::Mat<int> ibdRecHistParChrPar1 = ibdRecHist.getHist(pId, chr, 0);
-              arma::Mat<int> ibdRecHistParChrPar2 = ibdRecHist.getHist(pId, chr, 1);
-              arma::field<arma::Mat<int> > ibdRecHistParChrPar(2);
-              ibdRecHistParChrPar(0) = ibdRecHistParChrPar1;
-              ibdRecHistParChrPar(1) = ibdRecHistParChrPar2;
-              arma::uvec nIbdRecSegParChrPar(2);
-              nIbdRecSegParChrPar(0) = ibdRecHistParChrPar(0).n_rows;
-              nIbdRecSegParChrPar(1) = ibdRecHistParChrPar(1).n_rows;
-              
-              // Find and advance the ancestral recombinations in line with the recent (parent-progeny) recombinations
-              arma::uvec ibdRecSegPar(2);
-              int nIbdSegInd;
-              arma::Mat<int> ibdRecHistIndChrPar;
-              for (arma::uword run = 0; run < 2; ++run) {
-                if (run != 0) {
-                  ibdRecHistIndChrPar.set_size(nIbdSegInd, 2);
-                }
-                ibdRecSegPar(0) = 0;
-                ibdRecSegPar(1) = 0;
-                nIbdSegInd = 0;
-                for (arma::uword recSegInd = 0; recSegInd < nRecSegInd; ++recSegInd) {
-                  int source = recHistIndChrPar(recSegInd, 0) - 1;
-                  int startInd = recHistIndChrPar(recSegInd, 1);
-                  int stopInd;
-                  if (recSegInd == (nRecSegInd - 1)) {
-                    stopInd = nLociPerChr(chr);
-                  } else {
-                    stopInd = recHistIndChrPar(recSegInd + 1, 1) - 1;
-                  }
-
-                  bool loop = true;
-                  while (loop & (ibdRecSegPar(source) < nIbdRecSegParChrPar(source))) {
-                    int sourcePar = ibdRecHistParChrPar(source)(ibdRecSegPar(source), 0);
-                    int startPar  = ibdRecHistParChrPar(source)(ibdRecSegPar(source), 1);
-                    int stopPar;
-                    if (ibdRecSegPar(source) == (nIbdRecSegParChrPar(source) - 1)) {
-                      stopPar = nLociPerChr(chr);
-                    } else {
-                      stopPar = ibdRecHistParChrPar(source)(ibdRecSegPar(source) + 1, 1) - 1;
-                    }
-
-                    if (startInd <= stopPar) {
-                      if (stopInd >= startPar) {
-                        int startIbd = std::max(startInd, startPar);
-                        if (run == 1) {
-                          ibdRecHistIndChrPar(nIbdSegInd, 0) = sourcePar;
-                          ibdRecHistIndChrPar(nIbdSegInd, 1) = startIbd;
-                        }
-                        nIbdSegInd += 1;
-
-                        if (stopInd <= stopPar) {
-                          loop = false;
-                        }
-                        if ((stopInd >= stopPar) & (stopPar < nLociPerChr(chr))) {
-                          ibdRecSegPar(source) += 1;
-                        }
-                      } else {
-                        loop = false;
-                      }
-                    } else {
-                      ibdRecSegPar(source) += 1;
-                    }
-                  }
-                }
-              }
-              ibdRecHist.addHist(ibdRecHistIndChrPar, ind, chr, par);
-            }
-          }
-        }
-      }
-    }
-  }
-  return Rcpp::List::create(Rcpp::Named("ibdRecHist") = ibdRecHist.hist);
-}
